from code.objects import Wind, Bounce, Conveyor, Bit, Server, Computer, Menu, Data_Type, Settings, Wall, text
from code.calculations import direction, clamp_square
from code.static import *
from code.base_types import Screen_Object
from code.data import filepath
import pygame
import sys


class Music:
    def __init__(self, songs):
        self.counter = 0
        self.songs = []
        for song in songs:
            self.songs.append(filepath(song))
        pygame.mixer.music.load(self.songs[self.counter])
        pygame.mixer.music.play()
        self.counter += 1
        self.counter %= len(self.songs)
        
    def play(self):
        if not pygame.mixer.music.get_busy():
            pygame.mixer.music.load(self.songs[self.counter])
            pygame.mixer.music.play()
            self.counter += 1
            self.counter %= len(self.songs)


def total_data(objects):
    total = 0
    for object in objects:
        total += object.count_data()
    return total
    
    
def draw(screen, *groups):
    screen.fill(BLACK)
    for group in groups:
        #group.update()
        group.draw(screen)
        
def background(screen, game_pos):
    screen.fill(BLACK)
    rect = pygame.Rect((game_pos[0] - 2, game_pos[1] - 2), (DEFAULT_SCREEN_WIDTH - 200 + 4, DEFAULT_SCREEN_HEIGHT + 4))
    pygame.draw.rect(screen, FAINT_BLUE, rect)
    
        
def update_menu(screen, menu, total_width):
    menu.update()
    screen.blit(menu.image, (0, 0))


def screen_text(screen, string, color, time=3):
    
    clock = pygame.time.Clock()
    screen.fill(BLACK)
    success = text(string, 75, color)
    screen.blit(success, (screen.get_width()/2 - success.get_width()/2, screen.get_height()/2 - success.get_height()/2))
    i = 0
    while i < time * FPS: # i.e. for [time] seconds
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                sys.exit()
            if event.type == pygame.VIDEORESIZE:
                screen = pygame.display.set_mode((max(event.w, DEFAULT_SCREEN_WIDTH), max(event.h, DEFAULT_SCREEN_HEIGHT)), pygame.RESIZABLE)
                screen.blit(success, (screen.get_width()/2 - success.get_width()/2, screen.get_height()/2 - success.get_height()/2))
        pygame.display.update()
        clock.tick(FPS)
        i+=1
        
def click_screen(screen, string, color=LIGHT_GRAY, size=50):
    screen = pygame.display.set_mode(screen.get_size())
    
    string = string[0]
    temp = pygame.Surface(screen.get_size(), pygame.SRCALPHA)
    screen.blit(temp, (0, 0))
    success = text(string, size, color)
    click = text("Click to continue", 25, GRAY)
    screen.blit(success, (screen.get_width()/2 - success.get_width()/2, screen.get_height()/2 - success.get_height()/2))
    screen.blit(click, (screen.get_width()/2 - click.get_width()/2, screen.get_height()/2 - click.get_height()/2 + 50))
    
    exit = False
    while not exit:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                sys.exit()
            if event.type == pygame.KEYDOWN or event.type == pygame.MOUSEBUTTONDOWN:
                exit = True
        pygame.display.update()
        
        
def main():
    pygame.init()
    screen = pygame.display.set_mode((DEFAULT_SCREEN_WIDTH, DEFAULT_SCREEN_HEIGHT), pygame.RESIZABLE)
    music = Music(["Electrode Forest.ogg", "Peroxide.ogg", "sandwich pasta.ogg", "Never coming back.ogg", "Surface Tension.ogg", "Inverse.ogg", "Parallel.ogg"])
    # I'm too lazy to write a proper splash screen haha
    click_screen(screen, ["PACKETS"], WHITE, 100)
    levels = [
        {'interactive':{}, 'static':[('help','Get the packets (blue)...'),  ('help','from the server (red)...'), ('help','to the computer (green).'), ('help','Click "test level" to start'), ('server',(300,200)), ('computer',(300, 350))]},
        {'interactive':{'wind':1}, 'static':[('help','Drag elements from the right...'), ('help','to move the packets.'), ('server',(100,200)), ('computer',(500, 200)), ('wall',(200, 250, 'h', 300))]},
        {'interactive':{'bounce':1}, 'static':[('help','You can change an objects...'), ('help','properties in the settings...'), ('help','on the bottom.'), ('server',(100,10)), ('computer',(300, 200))]},
        {'interactive':{'wind':1, 'bounce':1}, 'static':[('help','Right click to change which...'), ('help','object is in settings.'), ('server',(100,10)), ('computer',(400, 40)), ('wall',(200, 0, 'v', 400))]},
        {'interactive':{'bounce':1}, 'static':[('help','A bounce pad will also act...'), ('help','as a tunnel, if aimed right.'), ('server',(200,200)), ('computer',(300, 200))]},
        {'interactive':{'conveyor':1}, 'static':[('server', (400,10)), ('computer',(100, 300)), ('wall',(300, 100, 'v', 250)), ('wall',(300, 400, 'v', 250))]},
        {'interactive':{'wind':2}, 'static':[('server',(300,350)), ('computer',(300, 200)), ('wall',(300, 275, 'h', 25))]},
        {'interactive':{'conveyor':1, 'bounce':1, 'wind':1}, 'static':[('server',(300,10)), ('computer',(300, 300)), ('wall',(300, 250, 'h', 400)), ('wall',(350, 300, 'v', 100)), ('wall',(250, 300, 'v', 100))]},
        {'interactive':{'conveyor':3, 'bounce':3}, 'static':[('server',(50,400)), ('computer',(50, 100)), ('wall',(0, 350, 'h', 700)), ('wall',(0, 150, 'h', 700)), ('wall',(500, 250, 'h', 700)), ('time',(1000))]},
        {'interactive':{'wind':2, 'bounce':4}, 'static':[('server',(200,200)), ('computer',(200, 300)), ('wall',(300, 250, 'h', 300)), ('wall',(300, 250, 'v', 300)), ('wall',(150, 250, 'v', 2000))]},
        {'interactive':{'bounce':3}, 'static':[('server',(100,50)), ('computer',(300, 300)), ('wall',(150, -50, 'v', 300)), ('wall',(250, 200, 'v', 320)), ('wall',(150, 400, 'v', 500)), ('wall',(300, 350, 'h', 120)), ('wall',(350, 310, 'v', 100))]},
        {'interactive':{'bounce':3, 'conveyor':3}, 'static':[('server',(50,10)), ('computer',(500, 200)), ('wall',(200, 0, 'v', 700)), ('wall',(400, 0, 'v', 700)), ('wall',(300, 400, 'v', 700)), ('time',1000)]},
        {'interactive':{'wind':5, 'bounce':2}, 'static':[('server',(50,10)), ('computer',(500, 200)), ('wall',(200, 0, 'v', 700)), ('wall',(400, 0, 'v', 700)), ('wall',(300, 400, 'v', 700)), ('time',1000)]},
    ]
    for level_data in levels:
        while not level(screen, level_data, music):
            screen_text(screen, "FAIL", RED)
        screen_text(screen, "SUCCESS", WHITE)
    
    screen_text(screen, "You the real winner", GOLD)
    
        
def level(screen, objects, music):
    screen = pygame.display.set_mode(screen.get_size(), pygame.RESIZABLE)
    game = pygame.Surface((screen.get_width()-MENU_WIDTH, screen.get_height()))
    # Where the game is rendered
    # Initially (0, 0) as it is the default screen size
    game_pos = ((screen.get_width()-200) / 2 - game.get_width() / 2, screen.get_height() / 2 - game.get_height() / 2)
    menu_surfece = pygame.Surface((MENU_WIDTH, DEFAULT_SCREEN_HEIGHT))
    
    # A group for everything
    all_group = pygame.sprite.Group()
    # A pygame group for things the user interacts with
    interaction_group = pygame.sprite.Group()
    # Pygame group for dynamic objects that the user can't interact with
    dynamic_group = pygame.sprite.Group()
    # Group for the individual objects
    bit_group = pygame.sprite.Group()
    server_group = pygame.sprite.Group()
    computer_group = pygame.sprite.Group()
    # A group for things that move the bits but can't be interacted with
    scene_group = pygame.sprite.Group()
    
    settings = Settings(screen.get_width(), [], None)
    
    not_testing = True
    
    Wind.containers = interaction_group, all_group
    Bounce.containers = interaction_group, all_group
    Conveyor.containers = interaction_group, all_group
    Bit.containers = dynamic_group, bit_group, all_group
    Server.containers = dynamic_group, server_group, all_group
    Computer.containers = dynamic_group, computer_group, all_group
    Wall.containers = dynamic_group, scene_group, all_group
    
    max_time = MAX_AGE
    
    help = []
    
    menu = Menu(screen.get_width(), screen.get_height(), objects['interactive'])
    for object in objects['static']:
        if object[0] == 'server':
            Server(*object[1])
        elif object[0] == 'computer':
            Computer(*object[1])
        elif object[0] == "wall":
            Wall(*object[1])
        elif object[0] == "time":
            max_time = object[1]
        elif object[0] == 'help':
            help.append(object[1])
            
    
    clock = pygame.time.Clock()
    
    safe_packets = 0
    
    sent_packets = 0
                
    # offset all items
    Screen_Object.offset = ((screen.get_width() - DEFAULT_SCREEN_WIDTH) / 2, (screen.get_height() - DEFAULT_SCREEN_HEIGHT) / 2)
    for object in all_group:
        object.move()
    
    
    game_pos = ((screen.get_width() - DEFAULT_SCREEN_WIDTH) / 2, (screen.get_height() - DEFAULT_SCREEN_HEIGHT) / 2)
    #print ("Game_pos =", game_pos)

    menu_surfece = pygame.Surface((MENU_WIDTH, screen.get_height()))
    menu.update_size(MENU_WIDTH, screen.get_height())
    settings.update_size(screen.get_width())
    
    while True:
        music.play()
        mouse_pos = list(pygame.mouse.get_pos())
        #mouse_pos[0] -= game_pos[0]
        #mouse_pos[1] -= game_pos[1]
        mouse_pressed = pygame.mouse.get_pressed()
        
        mouse_absolute_pos = pygame.mouse.get_pos()
        
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                sys.exit()
            if event.type == pygame.VIDEORESIZE:
                screen = pygame.display.set_mode((max(event.w, DEFAULT_SCREEN_WIDTH), max(event.h, DEFAULT_SCREEN_HEIGHT)), pygame.RESIZABLE)
                
                # offset all items
                Screen_Object.offset = ((screen.get_width()-200) / 2 - game.get_width() / 2, screen.get_height() / 2 - game.get_height() / 2)
                for object in all_group:
                    object.move()
                    
                # Resize the game screen
                game = pygame.Surface((max(event.w, DEFAULT_SCREEN_WIDTH)-MENU_WIDTH, max(event.h, DEFAULT_SCREEN_HEIGHT)))
                
                
                game_pos = ((screen.get_width()-200) / 2 - game.get_width() / 2, screen.get_height() / 2 - game.get_height() / 2)          
                #print ("Game_pos =", game_pos)

                menu_surfece = pygame.Surface((MENU_WIDTH, max(event.h, DEFAULT_SCREEN_HEIGHT)))
                menu.update_size(MENU_WIDTH, max(event.h, DEFAULT_SCREEN_HEIGHT))
                settings.update_size(screen.get_width())
            if event.type == pygame.KEYDOWN:
                pass
                
                
        menu.press(pygame.mouse.get_pos(), pygame.mouse.get_pressed()[0], screen.get_width())
                
        dynamic_group.update()
        for item in server_group:
            if item.timer == 0:
                if menu.testing:
                    if sent_packets < REQUIRED_PACKAGES:
                        sent_packets += 1
                        Bit(item.rect.centerx, item.rect.centery)
                else:
                    Bit(item.rect.centerx, item.rect.centery)
                
        
        for bit in bit_group:
            bit.move_pos(interaction_group, scene_group)
            if bit.age > max_time:
                bit.kill()
                if menu.testing:
                    return False
            for computer in computer_group:
                if bit.rect.colliderect(computer.rect):
                    if menu.testing:
                        safe_packets+=1
                        if safe_packets == REQUIRED_PACKAGES:
                            return True
                    bit.kill()
                    
                    
        if menu.testing and not_testing:
            not_testing = False
            for bit in bit_group:
                bit.kill()
                
        if not menu.testing:
            for item in interaction_group:
                item.carry(mouse_pos[0], mouse_pos[1], mouse_pressed[0])
                new_settings = item.right_click(mouse_pos[0], mouse_pos[1], mouse_pressed)
                if new_settings != None:
                    settings.set_items(new_settings, item)
            #item.rect.center = clamp_square(item.rect.center, game_pos, (game_pos[0] + DEFAULT_SCREEN_WIDTH - MENU_WIDTH, game_pos[1] + DEFAULT_SCREEN_HEIGHT))
            
        interaction_group.update()
        
        
        if not menu.testing:
            type = settings.click(mouse_absolute_pos[0], mouse_absolute_pos[1], mouse_pressed[0], screen.get_size())
            if type:
                menu.add(type)
                settings.set_items([], None)
            
            # All data objects (i.e. the things that the user controls) must be created here
            new_object = menu.create()
            if new_object != None:
                if new_object == "wind":
                    Wind(mouse_pos[0], mouse_pos[1])
                    #print (total_data(interaction_group))
                elif new_object == "bounce":
                    Bounce(mouse_pos[0], mouse_pos[1])
                    #print (total_data(interaction_group))
                elif new_object == "conveyor":
                    Conveyor(mouse_pos[0], mouse_pos[1])
                    #print (total_data(interaction_group))
            
            
        # Draw the things on the thing
        
        draw(game, interaction_group, dynamic_group)
        menu_surfece.fill((GREEN))
        if not menu.testing:
            update_menu(menu_surfece, menu, screen.get_width())
            
        screen.blit(menu_surfece, (screen.get_width()-MENU_WIDTH, 0))
        screen.blit(game, (0, 0))#game_pos)
        screen.blit(settings.image, (0, screen.get_height() - SETTINGS_HEIGHT))
        
        # Display the help screens
        pygame.display.update()
        for item in help:
            click_screen(screen, [item])
            screen = pygame.display.set_mode(screen.get_size(), pygame.RESIZABLE)
        help = []
        
        # Cap the FPS
        clock.tick(FPS)
    

if __name__ == '__main__':
    main()